/*
 * Log Levels: 
    debug   DEBUG: Info useful to developers for debugging the application, not useful during operations
    info    INFORMATIONAL: Normal operational messages - may be harvested for reporting, measuring throughput, etc - no action required
    notice  NOTICE: Events that are unusual but not error conditions - might be summarized in an email to developers or admins to spot potential problems - no immediate action required
    warning WARNING: Warning messages - not an error, but indication that an error will occur if action is not taken, e.g. file system 85% full - each item must be resolved within a given time
    error   ERROR: Non-urgent failures - these should be relayed to developers or admins; each item must be resolved within a given time
    alert   ALERT: Should be corrected immediately - notify staff who can fix the problem - example is loss of backup ISP connection
    crit    CRITICAL: Should be corrected immediately, but indicates failure in a primary system - fix CRITICAL problems before ALERT - example is loss of primary ISP connection
    emerg   EMERGENCY: A "panic" condition - notify all tech staff on call? (earthquake? tornado?) - affects multiple apps/servers/sites...
 
  Log format:
    '%(levelname)s %(appname)s %(instance)s [%(filename)s:%(lineno)d] %(message)s'
 */ 

/*
 * UWAGA!!!! - robienie console.log w tym pliku może grozic rekurencyjnym wywolaniem - jeśli LogConsole zostało użyte do przeciązenia console
 */

var Dgram = require("dgram");
var Glossy = require("glossy");
var Os = require("os");


var Stacktrace = require("stacktrace");
 
var socket = Dgram.createSocket("unix_dgram"); 


/** Temporary hack for node 0.4.12 - have to change this after switch to 0.6.x */
//var Util = require("util");
var util = require('util');
var Util = {};
var formatRegExp = /%[sdj]/g;
Util.format = function(f) {
  if (typeof f !== 'string') {
    var objects = [];
    for (var i = 0; i < arguments.length; i++) {
      objects.push(util.inspect(arguments[i]));
    }
    return objects.join(' ');
  }

  var i = 1;
  var args = arguments;
  var str = String(f).replace(formatRegExp, function(x) {
    switch (x) {
      case '%s': return String(args[i++]);
      case '%d': return Number(args[i++]);
      case '%j': return JSON.stringify(args[i++]);
      default:
        return x;
    }
  });
  for (var len = args.length, x = args[i]; i < len; x = args[++i]) {
    if (x === null || typeof x !== 'object') {
      str += ' ' + x;
    } else {
      str += ' ' + util.inspect(x);
    }
  }
  return str;
}


var logger = new Glossy.Produce({
    type: 'RFC5424',
    facility: 'local7'//, //domyslne
    //app_id: instanceId,
    //pid: process.pid
});

var accessLogger = new Glossy.Produce({
    type: 'RFC5424',
    facility: 'local0'//, //domyslne
    //app_id: instanceId,
    //pid: process.pid
});

var infoTypes = ['info', 'notice'];
var errorTypes = ['error', 'alert', 'crit', 'emerg'];

var Log = function(appId, level, message){
    var stackOffset = 2;
    var ret = Stacktrace.getStack(13);
    this.appId = appId ? appId : null;
    this.file = ret[stackOffset].scriptName;
    this.line = ret[stackOffset].lineNumber;
    this.stack = ret;
    this.level = level ? level : null;
    this.message = message ? message : "";

    this.getLogMessage = function(){
        var message = this.message;
        if(this.message instanceof Array){
            message = Util.format.apply(this, this.message) + "\n";
        }
        
        var stack = this._formatStack(this.stack);
        
        var msg = "[" + this.file + ":" + this.line + "] " + message + stack; 
        return logger.produce({
            facility: 'local7', //domyslne
            severity: this.level,
            host: Os.hostname(),
            app_id: this.appId,
            pid: process.pid,
            msg_id: "",
            message: msg
        });
    }
    
    this._formatStack = function(stackObj){
        if(errorTypes.indexOf(this.level) >= 0){
            var stack = "", i = stackOffset;
            for(i; i < stackObj.length; i++ ){
                stack += "\tat " + stackObj[i].functionName + " (" +  stackObj[i].scriptName + ":" + stackObj[i].lineNumber + ")\n";
            }
            return stack;
        }
        return "";
    };  
};
 
var LogConsole = function(appName, instanceId, version){
    var appName = appName;
    var instanceId = instanceId;
    var appVersion = version;
    var appId = appName + "@" + appVersion + ":" + instanceId;
         
    this.enableDebugLog = true;
        
    //debug
    this.log = function(){
        if(this.enableDebugLog){
            var logObj = new Log(appId, 'debug', Array.prototype.slice.call(arguments));
            this._log(logObj);
        }
    };
    //info
    //notice
    this.info = function(){
        if(arguments.length > 1 && arguments[0] == 'access') {
            var args = Array.prototype.slice.call(arguments)
            this._accessLog(this._shiftArguments(args));
        }
        else {
            var logObj = new Log(appId, 'debug', arguments);
            this._logTypes(logObj, infoTypes);   
        }
    };
    //warning
    this.warn = function(){
        var logObj = new Log(appId, 'warning', Array.prototype.slice.call(arguments));
        this._log(logObj);
    };
    //error
    //alert
    //crit
    //emerg
    this.error = function(){
        var logObj = new Log(appId, 'error', arguments);
        this._logTypes(logObj, errorTypes);
    };
    
    //not implemented
    this.dir = function(){};
    this.time = function(){};
    this.timeEnd = function(){};
    this.trace = function(){};
    this.assert = function(){};

    this._log = function(logObj){
        var data = new Buffer(logObj.getLogMessage());
        socket.send(data, 0, data.length, "/dev/log");
    };
    this._logTypes = function(logObj, types){
        args = Array.prototype.slice.call(logObj.message)
        if(args.length > 1 && typeof(args[0]) == 'string'){
            if(types.indexOf(args[0]) >= 0){
                logObj.level = args[0];
                logObj.message = this._shiftArguments(args);
                this._log(logObj);
                return;
            }
        }
        logObj.level = types[0];
        logObj.message = args;
        this._log(logObj);        
    };
    this._accessLog = function(message){
        message = Util.format.apply(this, message);
        message = accessLogger.produce({
            facility: 'local0', //domyslne
            severity: 'info',
            host: Os.hostname(),
            app_id: appId,
            pid: process.pid,
            msg_id: "",
            message: message
        });
        var data = new Buffer(message);
        socket.send(data, 0, data.length, "/dev/log");
    };
    this._shiftArguments = function(args){
        return args.slice(1, args.length); 
    };
};
exports.LogConsole = LogConsole;